package com.lqkj.cmlcp.module.authority.controller;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.lqkj.cmlcp.message.Result;
import com.lqkj.cmlcp.module.authority.domain.UserInfo;
import com.lqkj.cmlcp.module.authority.service.UserInfoService;
import com.lqkj.cmlcp.module.jwt.service.JwtService;
import com.lqkj.cmlcp.utils.AESUtils;
import com.lqkj.cmlcp.utils.RSAUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


@RestController
@RequestMapping("/user")
@Tag(name = "用户管理", description = "用户管理")
public class UserInfoController {

    @Autowired
    private UserInfoService userInfoService;

    @Autowired
    private JwtService jwtService;

    @Operation(
            summary = "5.1.1.3 获取登录用户接口",
            description = "5.1.1.3 获取登录用户接口"
    )
    @PostMapping("/detail")
    public Result<UserInfo> detail(HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String username = jwtService.decryptUsernameWithHeader(authHeader);
        return Result.ok(userInfoService.detailByUserCode(username), "获取登录用户");
    }

    @Operation(
            summary = "5.1.3.24 获取管理菜单接口",
            description = "5.1.3.24 获取管理菜单接口"
    )
    @PostMapping("/authList")
    public Result authList(HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String username = jwtService.decryptUsernameWithHeader(authHeader);
        return Result.ok(userInfoService.findAuthListWithUserCode(username), "获取管理菜单接口");
    }

    @Operation(
            summary = "5.1.3.11 用户分页接口",
            description = "5.1.3.11 用户分页接口",
            parameters = {
                    @Parameter(name = "userCode", description = "用户账号"),
                    @Parameter(name = "displayName", description = "用户名"),
                    @Parameter(name = "page", description = "页码"),
                    @Parameter(name = "pageSize", description = "每页数据条数"),
                    @Parameter(name = "key", description = "RSA加密后的aesKey")
            }
    )
    @PostMapping("/manage/pageQuery")
    public Result<JSONObject> pageQuery(@RequestParam(required = false, defaultValue = "") String userCode,
                                                 @RequestParam(required = false, defaultValue = "") String displayName,
                                                 @RequestParam(required = false, defaultValue = "0") Integer page,
                                                 @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                 @RequestParam String key) throws Exception {
        Page<UserInfo> userInfoPage = userInfoService.pageQuery(userCode, displayName, page, pageSize);
        String aesKey = RSAUtils.decryptBase64(key);
        List<UserInfo> list = new ArrayList<>();
        for (UserInfo userInfo : userInfoPage.getContent()) {
            list.add(AESUtils.encryptUser(userInfo, aesKey));
        }
        JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(userInfoPage));
        jsonObject.put("content", list);
        return Result.ok(jsonObject, "用户分页接口");
    }

    @Operation(
            summary = "5.1.3.28 用户详情接口",
            description = "5.1.3.28 用户详情接口",
            parameters = {
                    @Parameter(name = "userId", required = true, description = "用户ID"),
                    @Parameter(name = "key", description = "RSA加密后的aesKey")
            }
    )
    @PostMapping("/manage/detail")
    public Result<UserInfo> detailById(@RequestParam Integer userId,
                                            @RequestParam String key) throws Exception {
        String aesKey = RSAUtils.decryptBase64(key);
        return Result.ok(AESUtils.encryptUser(userInfoService.detail(userId), aesKey), "用户详情接口");
    }

    @Operation(
            summary = "5.1.3.12 保存用户接口",
            description = "5.1.3.12 保存用户接口",
            requestBody = @io.swagger.v3.oas.annotations.parameters.RequestBody(
                    description = "用户实体",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = UserInfo.class)
                    )
            )
    )
    @PostMapping("/manage/save")
    public Result save(@RequestBody UserInfo userInfo) {
        userInfo.setUserCode(RSAUtils.decryptBase64(userInfo.getUserCode()));
        if (StringUtils.isNotBlank(userInfo.getPassword()) && userInfo.getPassword().length() == 172) {
            userInfo.setPassword(RSAUtils.decryptBase64(userInfo.getPassword()));
        }
//        userInfo.setAuthorizationCode(RSAUtils.decryptBase64(userInfo.getAuthorizationCode()));
        String message = userInfoService.save(userInfo);
        if (message == null) return Result.ok(null, "保存用户接口");
        return Result.fail(message);
    }

    @Operation(
            summary = "5.1.3.13 删除用户接口",
            description = "5.1.3.13 删除用户接口",
            parameters = {
                    @Parameter(name = "userIds", required = true, description = "用户ID列表")
            }
    )
    @PostMapping("/manage/delete")
    public Result delete(@RequestParam List<Integer> userIds) {
        userInfoService.delete(userIds);
        return Result.ok(null, "删除用户接口");
    }


    @Operation(
            summary = "5.1.2.13 用户设置",
            description = "5.1.2.13 用户设置",
            parameters = {
                    @Parameter(name = "displayName", description = "用户名称"),
                    @Parameter(name = "headImgPath", description = "头像地址")
            }
    )
    @PostMapping( "/updateInfo")
    public Result<String> saveUser(@RequestParam(required = false) String displayName,
                                         @RequestParam(required = false) String headImgPath,
                                         HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String userCode = jwtService.decryptUsernameWithHeader(authHeader);
        userInfoService.updateInfo(userCode, displayName, headImgPath);
        return Result.ok(null, "更新用户信息");
    }

    /**
     * 检查是否能登录后台
     */
    @GetMapping("/check")
    public Result<String> check(String userCode) {
        if (userInfoService.check(userCode)){
            return Result.ok("可以登录后台");
        }else{
            return Result.fail("暂无权限，请联系管理员");
        }

    }

}
