package com.lqkj.cmlcp.module.authority.service.impl;

import com.lqkj.cmlcp.module.authority.domain.UserInfo;
import com.lqkj.cmlcp.module.authority.repository.RoleInfoRepository;
import com.lqkj.cmlcp.module.authority.repository.UserInfoRepository;
import com.lqkj.cmlcp.module.authority.service.UserInfoService;
import jakarta.annotation.PostConstruct;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
public class UserInfoServiceImpl implements UserInfoService{


    @Autowired
    private RoleInfoRepository roleInfoRepository;

    @Autowired
    private UserInfoRepository userInfoRepository;

    @Lazy
    @Autowired
    private PasswordEncoder passwordEncoder;

    @Override
    public UserInfo findByUserCode(String userCode) {

        return userInfoRepository.findByUserCode(userCode);
    }

        /**
     * 账号是否锁定
     * @param userCode
     * @return
     */
    @Transactional
    public boolean isLocked(String userCode) {
        UserInfo userInfo = findByUserCode(userCode);
        if (userInfo == null){
            return false;
        }else {
            if (userInfo.getLocking()) {
                if (new Date().getTime() - userInfo.getLockTime().getTime() > 1000 * 60 * 60){
                    userInfoRepository.cleanLockStatus(userInfo.getId());
                    return false;
                }else{
                    return true;
                }
            }else{
                return false;
            }
        }
    }

    /**
     * 更新登录错误次数
     * @param userCode
     * @return
     */
    @Transactional
    public String lockedUser(String userCode){
        UserInfo userInfo = findByUserCode(userCode);
        if (userInfo == null){
            return "用户名或密码错误";
        }else{
            int loginErrorCount = userInfo.getLoginErrorCount() == null ? 0 : userInfo.getLoginErrorCount();
            loginErrorCount += 1;
            userInfo.setLoginErrorCount(loginErrorCount);
            if (loginErrorCount == 4){
                userInfo.setLocking(true);
                userInfo.setLockTime(new Date());
                userInfoRepository.save(userInfo);
                return "账号已被冻结,请稍后重试";
            }
            userInfoRepository.save(userInfo);
            return "账号、密码输入错误," + (4 - loginErrorCount) +"次错误后,账号将会冻结1小时";
        }
    }

    /**
     * 解锁用户
     * @param userCode
     */
    @Transactional
    public void unlockedUser(String userCode){
        UserInfo userInfo = findByUserCode(userCode);
        userInfoRepository.cleanLockStatus(userInfo.getId());
    }

    @Override
    public UserInfo selectByUserName(String username) {
        return findByUserCode(username);
    }

    @Override
    public List<Integer> findUserRole(Integer userId) {

        return userInfoRepository.findUserRole(userId);
    }

    @Override
    public UserInfo detailByUserCode(String userCode) {
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        userInfo.setHasAuth(userInfoRepository.hasAuth(userInfo.getId()));
        return userInfo;
    }

    @Override
    public List<Map<String, Object>> findAuthListWithUserCode(String userCode) {
        List<Map<String, Object>> resultList = new ArrayList<>();
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        if (userInfo.getHasManage()) {
            List<Map<String, Object>> authList = userInfoRepository.queryAuthWithUserCode(userCode);

            Map<Integer, List<Map<String, Object>>> childrenMap = new HashMap<>();
            for (Map<String, Object> auth : authList) {
                Map<String, Object> result = new HashMap<>(auth);
                Map<String, Object> metaObject = new HashMap<>();
                metaObject.put("icon", auth.get("icon"));
                metaObject.put("title", auth.get("title"));
                result.put("meta", metaObject);

                if (auth.get("parent_id") == null) {
                    result.put("children", childrenMap.get(auth.get("authority_id")));
                    resultList.add(result);
                } else {
                    List<Map<String, Object>> childrenList = childrenMap.getOrDefault(auth.get("parent_id"), new ArrayList<>());
                    childrenList.add(result);
                    childrenMap.put((Integer) auth.get("parent_id"), childrenList);
                }
            }
        }

        return resultList;
    }

    @Override
    public Page<UserInfo> pageQuery(String userCode, String displayName, Integer page, Integer pageSize) {
        Pageable pageable = PageRequest.of(page, pageSize);
        Page<UserInfo> userInfos = userInfoRepository.pageQueryV2(userCode, displayName, pageable);
        for (UserInfo userInfo : userInfos.getContent()) {
            userInfo.setRoleNames(roleInfoRepository.findRoleNames(userInfo.getId()));
        }
        return userInfos;
    }

    @Override
    public UserInfo detail(Integer userId) {
        UserInfo userInfo = userInfoRepository.findById(userId).get();
        userInfo.setRoleInfoList(roleInfoRepository.listWithUserId(userId));
        return userInfo;
    }

    @Transactional
    @Override
    public String save(UserInfo userInfo) {

        // 账号判断，8位数字
        String userCodeRegex = "\\d{8}";
        Pattern pattern = Pattern.compile(userCodeRegex);
        Matcher matcher = pattern.matcher(userInfo.getUserCode());
        if (!matcher.find()) {
            return "账号必须是8位数字！";
        }
        // 显示名称判断，2-8位中文字符
        String displayNameRegex = "[\u4E00-\u9FA5]{2,8}";
        pattern = Pattern.compile(displayNameRegex);
        matcher = pattern.matcher(userInfo.getDisplayName());
        if (!matcher.find()) {
            return "名称必须是2-8位中文字符！";
        }

        // 授权码判断，8位数字、大写字母、小写字母
//        String auCodeRegex = "^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9]).{8}$";
//        pattern = Pattern.compile(auCodeRegex);
//        matcher = pattern.matcher(userInfo.getAuthorizationCode());
//        if (!matcher.find()) {
//            return "授权码必须是8位数字、大写字母、小写字母！";
//        }

        if (userInfo.getId() == null) {
            // 重复判断
            if (userInfoRepository.hasSameUserCode(userInfo.getUserCode())) {
                return "账号与已有账号重复，请设置其他账号！";
            }
            // 密码判断，8-16位数字、大写字母、小写字母
            String passwordRegex = "^(?=.*\\d)(?=.*[a-z])(?=.*[A-Z])[a-zA-Z0-9]{8,16}$";
            pattern = Pattern.compile(passwordRegex);
            matcher = pattern.matcher(userInfo.getPassword());
            if (!matcher.find()) {
                return "密码必须是8-16位数字、大写字母、小写字母！";
            }

            userInfo.setLocking(false);
            userInfo.setPassword(passwordEncoder.encode(userInfo.getPassword()));
            userInfo.setUpdateTime(new Date());
            userInfo = userInfoRepository.save(userInfo);
        } else {
            if (StringUtils.isNotBlank(userInfo.getPassword())) {
                // 密码判断，8-16位数字、大写字母、小写字母
                String passwordRegex = "^(?=.*\\d)(?=.*[a-z])(?=.*[A-Z])[a-zA-Z0-9]{8,16}$";
                pattern = Pattern.compile(passwordRegex);
                matcher = pattern.matcher(userInfo.getPassword());
                if (!matcher.find()) {
                    return "密码必须是8-16位数字、大写字母、小写字母！";
                }
            }
            UserInfo oldUser = userInfoRepository.findById(userInfo.getId()).get();
            if (StringUtils.isNotBlank(userInfo.getPassword())) {
                oldUser.setPassword(passwordEncoder.encode(userInfo.getPassword()));
            }
            oldUser.setDisplayName(userInfo.getDisplayName());
            oldUser.setHasManage(userInfo.getHasManage());
//            oldUser.setAuthorizationCode(userInfo.getAuthorizationCode());
            oldUser.setUpdateTime(new Date());
            userInfoRepository.save(oldUser);
        }
//        roleInfoRepository.updateUserRole(userInfo.getId(), userInfo.getRoleIds());
        return null;
    }

    @Override
    public void delete(List<Integer> userIds) {
        userInfoRepository.deleteAllByIdInBatch(userIds);
    }

    @Override
    public void updateInfo(String userCode, String displayName, String headImgPath) {
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        if (StringUtils.isNotBlank(displayName)) {
            userInfo.setDisplayName(displayName);
        }
        if ((StringUtils.isNotBlank(headImgPath))) {
            userInfo.setHeadImg(headImgPath);
        }
        userInfoRepository.save(userInfo);
    }

    @Override
    public boolean check(String userCode) {
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        if (userInfo == null) {
            throw new RuntimeException("账号或密码错误！");
        }
        return userInfo.getHasManage();
    }

    @PostConstruct
    @Transactional
    public void initSuperUser() {
        if (!userInfoRepository.existsById(1)) {
            userInfoRepository.initSuperAdmin();
            userInfoRepository.initUserIdSeq();
            userInfoRepository.initSuperAdminRole();
        }
    }



}