package com.lqkj.cmlcp.module.base.controller;


import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyEmitter;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;

@RestController
@RequestMapping("/chat")
public class ChatController {

    private static final ConcurrentHashMap<String, ResponseBodyEmitter> onlineUsers = new ConcurrentHashMap<>();
    // 存储聊天消息（可替换为数据库）
    private static final List<Map<String, String>> messageHistory = new CopyOnWriteArrayList<>();

    /**
     * 用户连接聊天通道
     * @param userId 用户唯一标识
     */
    @GetMapping("/connect")
    public ResponseBodyEmitter connect(@RequestParam String userId) {

        ResponseBodyEmitter emitter = new ResponseBodyEmitter(-1L);
        // 移除旧连接并添加新连接
        onlineUsers.computeIfPresent(userId, (k, v) -> {
            v.complete();
            return emitter;
        });
        onlineUsers.putIfAbsent(userId, emitter);

        // 发送历史消息（可选）
        sendExistingMessages(emitter);

        // 设置连接关闭/超时的清理逻辑
        emitter.onCompletion(() -> onlineUsers.remove(userId));
        emitter.onTimeout(() -> onlineUsers.remove(userId));
        emitter.onError((e) -> onlineUsers.remove(userId));

        return emitter;

    }

    /**
     * 发送消息接口
     * @param message 消息体（JSON格式）
     */
    @PostMapping("/send")
    public ResponseEntity<String> sendMessage(@RequestBody Map<String, String> message) {
        String sender = message.get("sender");
        String receiver = message.get("receiver");
        String content = message.get("content");

        if (sender == null || content == null) {
            return ResponseEntity.badRequest().body("参数缺失");
        }

        // 构造消息对象
        Map<String, String> msg = new HashMap<>();
        msg.put("sender", sender);
        msg.put("receiver", receiver != null ? receiver : "all"); // 默认群发
        msg.put("content", content);
        msg.put("timestamp", LocalDateTime.now().toString());

        // 存储消息
        messageHistory.add(msg);

        // 推送消息
        if (receiver == null) {
            // 群发：遍历所有在线用户
            onlineUsers.forEach((userId, emitter) -> {
                try {
                    emitter.send(msg);
                } catch (Exception e) {
                    emitter.complete();
                    onlineUsers.remove(userId);
                }
            });
        } else {
            // 私聊：查找指定接收者
            ResponseBodyEmitter receiverEmitter = onlineUsers.get(receiver);
            if (receiverEmitter != null) {
                try {
                    receiverEmitter.send(msg);
                } catch (Exception e) {
                    receiverEmitter.complete();
                    onlineUsers.remove(receiver);
                }
            }
        }

        return ResponseEntity.ok("消息已发送");
    }

    // 发送历史消息给新连接的用户
    private void sendExistingMessages(ResponseBodyEmitter emitter) {
        try {
            for (Map<String, String> msg : messageHistory) {
                emitter.send(msg);
            }
        } catch (Exception e) {
            emitter.complete();
        }
    }
}
